#######################################################################################
########## Code provided by Marine Brieuc 2015.
########## This code is provided as is
########## For any question, contacte Marine Brieuc at mbrieuc @ uw.edu
#######################################################################################


library(randomForest)

########### Genotypes imputed with FASTPhase
dat_imp = read.table("PS_CR_MAF10_imputed.txt", header=TRUE)
######### Peak_run_month and Corrected_Peak_run_month to get rid of population structure 
Run_data<-read.table("Run_month_and_Adjusted_peak_run_month.txt",sep="\t",header=TRUE)
colnames(Run_data)[1]="Individual"

All_data_imp<-merge(Run_data,dat_imp,by="Individual")

### Admixture proportions - BAPS output
run_time_admixture_dat<-read.table("CR_PS_MAF10_MV50_Admixture_proportion_Mean_run_time.txt",sep="\t",header=TRUE)
All_data_imp_with_adm<-merge(run_time_admixture_dat[,-2],All_data_imp,by="Individual")

#dat = dat[,-c(1,2,3)] # drop pop / lineage / run
gen_for_correc_PS_imp_DD=All_data_imp_with_adm[,-c(1:10)]

####################### Recoding necessary to run regressions

convNA = function(x) {x[which(x=="-")]=NA; return(x)}
conv0 = function(x) {x[which(x=="a")]=0; return(x)}
conv1 = function(x) {x[which(x=="ab")]=1; return(x)}
conv2 = function(x) {x[which(x=="b")]=2; return(x)}
convNum = function(x) {x = as.numeric(x)}
convFac = function(x) {x = as.factor(x); return(x)}

# Convert genotypes to numbers to run regression
gen_for_correc_PS_imp_DD = apply(gen_for_correc_PS_imp_DD,2,convNA)
gen_for_correc_PS_imp_DD = apply(gen_for_correc_PS_imp_DD,2,conv0)
gen_for_correc_PS_imp_DD = apply(gen_for_correc_PS_imp_DD,2,conv1)
gen_for_correc_PS_imp_DD = apply(gen_for_correc_PS_imp_DD,2,conv2)
gen_for_correc_PS_imp_DD = apply(gen_for_correc_PS_imp_DD,2,convNum)
gen_for_correc_PS_imp_DD[1:10,1:10]

####### Correction for population structure (PS as Structure classification)
corr_genotypes_imputed_DD<-data.frame(Individual=All_data_imp_with_adm[,1])
for (i in 1:ncol(gen_for_correc_PS_imp_DD)){
  LM_SNP_i <- lm(gen_for_correc_PS_imp_DD[,i] ~ All_data_imp_with_adm$Lineage1 + All_data_imp_with_adm$Lineage2 + All_data_imp_with_adm$Lineage4,na.action=na.exclude) 
  corr_genotypes_imputed_DD[,i+1] <- gen_for_correc_PS_imp_DD[,i] - predict(LM_SNP_i, All_data_imp_with_adm)
  colnames(corr_genotypes_imputed_DD)[i+1]<-colnames(gen_for_correc_PS_imp_DD)[i] 
  if(i%%50==0) print(i)
  #head(new_dat)
}



####RF with all 9107 loci All lineages all populations ################
rf_Peak_run_month_corrected_gen_corrected_PS_CR_imputed_DD_10000_1 = randomForest(x = corr_genotypes_imputed_DD[,-1], y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
rf_Peak_run_month_corrected_gen_corrected_PS_CR_imputed_DD_10000_2 = randomForest(x = corr_genotypes_imputed_DD[,-1], y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
rf_Peak_run_month_corrected_gen_corrected_PS_CR_imputed_DD_10000_3 = randomForest(x = corr_genotypes_imputed_DD[,-1], y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)

varImpPlot(rf_Peak_run_month_corrected_gen_corrected_PS_CR_imputed_DD_10000_1, main="All RunMonth ntree=10000 1",n.var=50)
varImpPlot(rf_Peak_run_month_corrected_gen_corrected_PS_CR_imputed_DD_10000_2, main="All RunMonth ntree=10000 2",n.var=50)
varImpPlot(rf_Peak_run_month_corrected_gen_corrected_PS_CR_imputed_DD_10000_3, main="All RunMonth ntree=10000 3",n.var=50)

## create object with all the importance values for each random forest run
imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_1<-data.frame(importance(rf_Peak_run_month_corrected_gen_corrected_PS_CR_imputed_DD_10000_1,type=1))
rownames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_1)<-colnames(corr_genotypes_imputed_DD[-1])
colnames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_1)<-c("importance")

imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_2<-data.frame(importance(rf_Peak_run_month_corrected_gen_corrected_PS_CR_imputed_DD_10000_2,type=1))
rownames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_2)<-colnames(corr_genotypes_imputed_DD[-1])
colnames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_2)<-c("importance")

imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_3<-data.frame(importance(rf_Peak_run_month_corrected_gen_corrected_PS_CR_imputed_DD_10000_3,type=1))
rownames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_3)<-colnames(corr_genotypes_imputed_DD[-1])
colnames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_3)<-c("importance")

##### Best 1% 
names_best_1perc_gen_RM_corr_1<-rownames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_1)[which(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_1$importance > quantile(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_1$importance, probs=0.99))]  # name of best 1% loci
names_best_1perc_gen_RM_corr_2<-rownames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_2)[which(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_2$importance > quantile(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_2$importance, probs=0.99))]  # name of best 1% loci
names_best_1perc_gen_RM_corr_3<-rownames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_3)[which(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_3$importance > quantile(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_3$importance, probs=0.99))]  # name of best 1% loci
names_best_1perc_gen_RM_corr_unique<-unique(c(names_best_1perc_gen_RM_corr_1,names_best_1perc_gen_RM_corr_2,names_best_1perc_gen_RM_corr_3))
corr_genotypes_imputed_DD_gen_RM_corr_best1perc<-corr_genotypes_imputed_DD[,colnames(corr_genotypes_imputed_DD) %in% names_best_1perc_gen_RM_corr_unique]
rf_Peak_run_month_PS_CR_imputed_DD_10000_best1perc_gen_RM_corr_1 = randomForest(x = corr_genotypes_imputed_DD_gen_RM_corr_best1perc, y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
rf_Peak_run_month_PS_CR_imputed_DD_10000_best1perc_gen_RM_corr_2 = randomForest(x = corr_genotypes_imputed_DD_gen_RM_corr_best1perc, y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
rf_Peak_run_month_PS_CR_imputed_DD_10000_best1perc_gen_RM_corr_3 = randomForest(x = corr_genotypes_imputed_DD_gen_RM_corr_best1perc, y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)


## I repeated that for the best 5%, 10%, 20%, 30%, 0.5%, 0.6%, 0.7%, 0.8%, 0.9%, 1.1%, 1.2%, 1.3%, 1.4%, 1.5%, 2%, 3%, 4%
## Then I collected all the PVE for each run and plotted it to determine where to start backward purging approach: 150

#################### Backward purging approach - start with 150 best loci - All Lineages #########

## get names for best 150 loci
names_best_150_gen_RM_corr_1<-rownames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_1)[which(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_1$importance > quantile(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_1$importance, probs=0.9853))]  # name of best 150 loci
names_best_150_gen_RM_corr_2<-rownames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_2)[which(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_2$importance > quantile(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_2$importance, probs=0.9853))]  # name of best 150 loci
names_best_150_gen_RM_corr_3<-rownames(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_3)[which(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_3$importance > quantile(imp_rf_Peak_run_month_PS_CR_imputed_DD_10000_3$importance, probs=0.9853))]  # name of best 150 loci
names_best_150_gen_RM_corr_unique<-unique(c(names_best_150_gen_RM_corr_1,names_best_150_gen_RM_corr_2,names_best_150_gen_RM_corr_3))

names_best150<-names_best_150_gen_RM_corr_unique

## extract genotypes for these loci
corr_genotypes_purging<-corr_genotypes_imputed_DD[,colnames(corr_genotypes_imputed_DD) %in% names_best150]

## run 3 iterations of random forest with these loci
rf_purging_1 = randomForest(x = corr_genotypes_purging, y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
rf_purging_2 = randomForest(x = corr_genotypes_purging, y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
rf_purging_3 = randomForest(x = corr_genotypes_purging, y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
## initiate backward purging 
names_all_iterations<-list()
names_all_iterations[[150]]<-names_best150
all_Var_explained<-data.frame(V1=1:150,V2=1:150,V3=1:150)
rownames(all_Var_explained)<-1:150
all_Var_explained[150,]<-c(mean(rf_purging_1$rsq),mean(rf_purging_2$rsq),mean(rf_purging_3$rsq))
all_mse<-data.frame(V1=1:150,V2=1:150,V3=1:150)
rownames(all_mse)<-1:150
all_mse[150,]<-c(mean(rf_purging_1$mse),mean(rf_purging_2$mse),mean(rf_purging_3$mse))
for (2 in 1:149){
  print(i)
  imp_purging_1<-data.frame(importance(rf_purging_1,type=1))
  imp_purging_2<-data.frame(importance(rf_purging_2,type=1))
  imp_purging_3<-data.frame(importance(rf_purging_3,type=1))
  rownames(imp_purging_1)<-colnames(corr_genotypes_purging)
  rownames(imp_purging_2)<-colnames(corr_genotypes_purging)
  rownames(imp_purging_3)<-colnames(corr_genotypes_purging)
  all_imp<-cbind(imp_purging_1,imp_purging_2,imp_purging_3)
  ##remove locus with lowest average importance value -- 
  all_imp$average<-apply(all_imp[,1:3],1,mean)
  dont_keep<-which(all_imp[,'average']==min(all_imp[,'average'])) ## remove least important
  if (length(dont_keep)==1) {
    table_keep<-all_imp[-dont_keep,] ## record the name of lovus removed
  } else {
    table_keep<-all_imp[-dont_keep[sample(x=dont_keep,n=1),]] ## if 2 loci are as unimportant, pick one
  }
  names_keep<-rownames(table_keep)
  names_all_iterations[[150-i]]<-names_keep
  corr_genotypes_purging<-corr_genotypes_imputed_DD[,colnames(corr_genotypes_imputed_DD) %in% names_keep]
  rf_purging_1 = randomForest(x = corr_genotypes_purging, y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
  rf_purging_2 = randomForest(x = corr_genotypes_purging, y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
  rf_purging_3 = randomForest(x = corr_genotypes_purging, y = All_data_imp$Residual, importance=TRUE ,proximity=TRUE, ntree=10000)
  all_Var_explained[150-i,]<-c(mean(rf_purging_1$rsq),mean(rf_purging_2$rsq),mean(rf_purging_3$rsq))
  all_mse[150-i,]<-c(mean(rf_purging_1$mse),mean(rf_purging_2$mse),mean(rf_purging_3$mse))
  
}

all_Var_explained$Average<-apply(all_Var_explained,1,mean)

plot(all_Var_explained$Average)

which(all_Var_explained$Average==max(all_Var_explained$Average))

names_all_iterations_df <- data.frame(matrix(NA,nrow=150,ncol=150))
colnames(names_all_iterations_df)<-c(1:150)
for (i in 2:150) {
  names_all_iterations_df[1:i,i]<-names_all_iterations[unlist(i)]
}

write.table(names_all_iterations_df,"Names_loci_iterations_1_150.txt")



